@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2011-2013 Andrew M. Bishop
+ This file Copyright 2011-2014 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -82,11 +82,11 @@ void StartPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
  /* Open the file read-only */
 
- segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
+ segmentsx->fd=ReOpenFileBuffered(segmentsx->filename_tmp);
 
  /* Read the on-disk image */
 
- while(!ReadFile(segmentsx->fd,&segmentx,sizeof(SegmentX)))
+ while(!ReadFileBuffered(segmentsx->fd,&segmentx,sizeof(SegmentX)))
    {
     index_t node1=segmentx.node1;
 
@@ -108,7 +108,7 @@ void StartPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
  /* Close the file */
 
- segmentsx->fd=CloseFile(segmentsx->fd);
+ segmentsx->fd=CloseFileBuffered(segmentsx->fd);
 
  /* Print the final message */
 
@@ -150,12 +150,12 @@ void FinishPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
 void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distance_t minimum)
 {
+ WaysX *newwaysx;
+ WayX tmpwayx;
  transport_t transport;
  BitMask *connected,*region;
  index_t *regionsegments,*othersegments;
- int nallocregionsegments,nallocothersegments;
- index_t nnewways=0;
- int fd;
+ index_t nallocregionsegments,nallocothersegments;
 
  if(nodesx->number==0 || segmentsx->number==0)
     return;
@@ -167,12 +167,19 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
  segmentsx->data=MapFileWriteable(segmentsx->filename_tmp);
  waysx->data=MapFile(waysx->filename_tmp);
 #else
- nodesx->fd=ReOpenFile(nodesx->filename_tmp);
- segmentsx->fd=ReOpenFileWriteable(segmentsx->filename_tmp);
- waysx->fd=ReOpenFile(waysx->filename_tmp);
+ nodesx->fd=SlimMapFile(nodesx->filename_tmp);
+ segmentsx->fd=SlimMapFileWriteable(segmentsx->filename_tmp);
+ waysx->fd=SlimMapFile(waysx->filename_tmp);
+
+ InvalidateNodeXCache(nodesx->cache);
+ InvalidateSegmentXCache(segmentsx->cache);
+ InvalidateWayXCache(waysx->cache);
 #endif
 
- fd=ReOpenFileWriteable(waysx->filename_tmp);
+ newwaysx=NewWayList(0,0);
+ CloseFileBuffered(newwaysx->fd);
+
+ newwaysx->fd=SlimMapFileWriteable(newwaysx->filename_tmp);
 
  connected=AllocBitMask(segmentsx->number);
  region   =AllocBitMask(segmentsx->number);
@@ -195,6 +202,9 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
     const char *transport_str=TransportName(transport);
     transports_t transports=TRANSPORTS(transport);
 
+    if(!(waysx->allow&transports))
+       continue;
+
     /* Print the start message */
 
     printf_first("Pruning Isolated Regions (%s): Segments=0 Adjusted=0 Pruned=0",transport_str);
@@ -206,10 +216,10 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
 
     for(i=0;i<segmentsx->number;i++)
       {
-       int nregionsegments=0,nothersegments=0;
+       index_t nregionsegments=0,nothersegments=0;
        distance_t total=0;
        SegmentX *segmentx;
-       WayX *wayx,tmpwayx;
+       WayX *wayx;
 
        if(IsBitSet(connected,i))
           goto endloop;
@@ -222,7 +232,7 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
        if(segmentx->way<waysx->number)
           wayx=LookupWayX(waysx,segmentx->way,1);
        else
-          SeekReadFile(fd,(wayx=&tmpwayx),sizeof(WayX),segmentx->way*sizeof(WayX));
+          SlimFetch(newwaysx->fd,(wayx=&tmpwayx),sizeof(WayX),(segmentx->way-waysx->number)*sizeof(WayX));
 
        if(!(wayx->way.allow&transports))
           goto endloop;
@@ -265,7 +275,7 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
                    if(segmentx->way<waysx->number)
                       wayx=LookupWayX(waysx,segmentx->way,1);
                    else
-                      SeekReadFile(fd,(wayx=&tmpwayx),sizeof(WayX),segmentx->way*sizeof(WayX));
+                      SlimFetch(newwaysx->fd,(wayx=&tmpwayx),sizeof(WayX),(segmentx->way-waysx->number)*sizeof(WayX));
 
                    if(wayx->way.allow&transports)
                      {
@@ -317,7 +327,7 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
              if(segmentx->way<waysx->number)
                 wayx=LookupWayX(waysx,segmentx->way,1);
              else
-                SeekReadFile(fd,(wayx=&tmpwayx),sizeof(WayX),segmentx->way*sizeof(WayX));
+                SlimFetch(newwaysx->fd,(wayx=&tmpwayx),sizeof(WayX),(segmentx->way-waysx->number)*sizeof(WayX));
 
              if(wayx->way.allow==transports)
                {
@@ -333,11 +343,11 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
 
                    tmpwayx.way.allow&=~transports;
 
-                   segmentx->way=waysx->number+nnewways;
+                   segmentx->way=waysx->number+newwaysx->number;
 
-                   SeekWriteFile(fd,&tmpwayx,sizeof(WayX),segmentx->way*sizeof(WayX));
+                   SlimReplace(newwaysx->fd,&tmpwayx,sizeof(WayX),(segmentx->way-waysx->number)*sizeof(WayX));
 
-                   nnewways++;
+                   newwaysx->number++;
 
                    PutBackSegmentX(segmentsx,segmentx);
                   }
@@ -345,7 +355,7 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
                   {
                    tmpwayx.way.allow&=~transports;
 
-                   SeekWriteFile(fd,&tmpwayx,sizeof(WayX),segmentx->way*sizeof(WayX));
+                   SlimReplace(newwaysx->fd,&tmpwayx,sizeof(WayX),(segmentx->way-waysx->number)*sizeof(WayX));
                   }
 
                 nadjusted++;
@@ -391,14 +401,26 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
  segmentsx->data=UnmapFile(segmentsx->data);
  waysx->data=UnmapFile(waysx->data);
 #else
- nodesx->fd=CloseFile(nodesx->fd);
- segmentsx->fd=CloseFile(segmentsx->fd);
- waysx->fd=CloseFile(waysx->fd);
+ nodesx->fd=SlimUnmapFile(nodesx->fd);
+ segmentsx->fd=SlimUnmapFile(segmentsx->fd);
+ waysx->fd=SlimUnmapFile(waysx->fd);
 #endif
 
- CloseFile(fd);
+ SlimUnmapFile(newwaysx->fd);
 
- waysx->number+=nnewways;
+ waysx->number+=newwaysx->number;
+
+ waysx->fd=OpenFileBufferedAppend(waysx->filename_tmp);
+
+ newwaysx->fd=ReOpenFileBuffered(newwaysx->filename_tmp);
+
+ while(!ReadFileBuffered(newwaysx->fd,&tmpwayx,sizeof(WayX)))
+    WriteFileBuffered(waysx->fd,&tmpwayx,sizeof(WayX));
+
+ CloseFileBuffered(waysx->fd);
+ CloseFileBuffered(newwaysx->fd);
+
+ FreeWayList(newwaysx,0);
 }
 
 
@@ -433,9 +455,13 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
  segmentsx->data=MapFileWriteable(segmentsx->filename_tmp);
  waysx->data=MapFile(waysx->filename_tmp);
 #else
- nodesx->fd=ReOpenFileWriteable(nodesx->filename_tmp);
- segmentsx->fd=ReOpenFileWriteable(segmentsx->filename_tmp);
- waysx->fd=ReOpenFile(waysx->filename_tmp);
+ nodesx->fd=SlimMapFileWriteable(nodesx->filename_tmp);
+ segmentsx->fd=SlimMapFileWriteable(segmentsx->filename_tmp);
+ waysx->fd=SlimMapFile(waysx->filename_tmp);
+
+ InvalidateNodeXCache(nodesx->cache);
+ InvalidateSegmentXCache(segmentsx->cache);
+ InvalidateWayXCache(waysx->cache);
 #endif
 
  /* Loop through the segments and find the short ones for possible modification */
@@ -567,7 +593,7 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
           WayX *wayx1,*wayx2,*wayx3;
           NodeX *nodex2,*nodex3,*newnodex;
           index_t newnode;
-          int join12=1,join23=1;
+          int join12=1,join23=1,same13=1;
 
           /* Check if pruning would collapse a loop */
 
@@ -668,10 +694,35 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
 
           PutBackNodeX(nodesx,newnodex);
 
-          /* Modify segments */
+          /* Modify segments - update the distances */
 
-          segmentx1->distance+=DISTANCE(segmentx2->distance)/2;
-          segmentx3->distance+=DISTANCE(segmentx2->distance)-DISTANCE(segmentx2->distance)/2;
+          if(!IsOneway(segmentx1) && !IsOneway(segmentx3))
+             ;
+          else if(IsOneway(segmentx1) && IsOneway(segmentx3))
+            {
+             if(IsOnewayTo(segmentx1,node3) && !IsOnewayFrom(segmentx3,node3)) /* S1 is one-way but S3 doesn't continue */
+                same13=0;
+
+             if(IsOnewayFrom(segmentx1,node3) && !IsOnewayTo(segmentx3,node3)) /* S1 is one-way but S3 doesn't continue */
+                same13=0;
+            }
+          else
+             same13=0;
+
+          if(WaysCompare(&wayx1->way,&wayx3->way))
+             same13=0;
+
+          if(same13)
+            {
+             segmentx1->distance+=DISTANCE(segmentx2->distance)/2;
+             segmentx3->distance+=DISTANCE(segmentx2->distance)-DISTANCE(segmentx2->distance)/2;
+            }
+          else if(join12)
+             segmentx1->distance+=DISTANCE(segmentx2->distance);
+          else /* if(join23) */
+             segmentx3->distance+=DISTANCE(segmentx2->distance);
+
+          /* Modify segments - update the segments */
 
           if(segmentx1->node1==node1)
             {
@@ -800,9 +851,9 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
  segmentsx->data=UnmapFile(segmentsx->data);
  waysx->data=UnmapFile(waysx->data);
 #else
- nodesx->fd=CloseFile(nodesx->fd);
- segmentsx->fd=CloseFile(segmentsx->fd);
- waysx->fd=CloseFile(waysx->fd);
+ nodesx->fd=SlimUnmapFile(nodesx->fd);
+ segmentsx->fd=SlimUnmapFile(segmentsx->fd);
+ waysx->fd=SlimUnmapFile(waysx->fd);
 #endif
 
  /* Print the final message */
@@ -827,8 +878,8 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
 {
  index_t i;
  index_t npruned=0;
+ index_t nalloc;
  BitMask *checked;
- int nalloc;
  index_t *nodes,*segments;
  double *lats,*lons;
  double maximumf;
@@ -836,8 +887,6 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
  if(nodesx->number==0 || segmentsx->number==0 || waysx->number==0)
     return;
 
- maximumf=distance_to_km(maximum);
-
  /* Print the start message */
 
  printf_first("Pruning Straight Highway Nodes: Nodes=0 Pruned=0");
@@ -849,9 +898,13 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
  segmentsx->data=MapFileWriteable(segmentsx->filename_tmp);
  waysx->data=MapFile(waysx->filename_tmp);
 #else
- nodesx->fd=ReOpenFile(nodesx->filename_tmp);
- segmentsx->fd=ReOpenFileWriteable(segmentsx->filename_tmp);
- waysx->fd=ReOpenFile(waysx->filename_tmp);
+ nodesx->fd=SlimMapFile(nodesx->filename_tmp);
+ segmentsx->fd=SlimMapFileWriteable(segmentsx->filename_tmp);
+ waysx->fd=SlimMapFile(waysx->filename_tmp);
+
+ InvalidateNodeXCache(nodesx->cache);
+ InvalidateSegmentXCache(segmentsx->cache);
+ InvalidateWayXCache(waysx->cache);
 #endif
 
  checked=AllocBitMask(nodesx->number);
@@ -872,6 +925,8 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
 
  /* Loop through the nodes and find stretches of simple highway for possible modification */
 
+ maximumf=distance_to_km(maximum);
+
  for(i=0;i<nodesx->number;i++)
    {
     int lowerbounded=0,upperbounded=0;
@@ -1110,7 +1165,7 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
          {
           SegmentX *segmentx;
           distance_t dist=0;
-          double dist1,dist2,dist3,dist3a,dist3b,distp;
+          double dist1,dist2,dist3,distp;
           index_t c;
 
           dist3=distance(lats[lower],lons[lower],lats[current],lons[current]);
@@ -1122,17 +1177,22 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
 
              /* Use law of cosines (assume flat Earth) */
 
-             dist3a=(dist1*dist1-dist2*dist2+dist3*dist3)/(2.0*dist3);
-             dist3b=dist3-dist3a;
-
-             if((dist1+dist2)<dist3)
+             if(dist3==0)
+                distp=dist1; /* == dist2 */
+             else if((dist1+dist2)<dist3)
                 distp=0;
-             else if(dist3a>=0 && dist3b>=0)
-                distp=sqrt(dist1*dist1-dist3a*dist3a);
-             else if(dist3a>0)
-                distp=dist2;
-             else /* if(dist3b>0) */
-                distp=dist1;
+             else
+               {
+                double dist3a=(dist1*dist1-dist2*dist2+dist3*dist3)/(2.0*dist3);
+                double dist3b=dist3-dist3a;
+
+                if(dist3a>=0 && dist3b>=0)
+                   distp=sqrt(dist1*dist1-dist3a*dist3a);
+                else if(dist3a>0)
+                   distp=dist2;
+                else /* if(dist3b>0) */
+                   distp=dist1;
+               }
 
              if(distp>maximumf) /* gone too far */
                 break;
@@ -1198,9 +1258,9 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
  segmentsx->data=UnmapFile(segmentsx->data);
  waysx->data=UnmapFile(waysx->data);
 #else
- nodesx->fd=CloseFile(nodesx->fd);
- segmentsx->fd=CloseFile(segmentsx->fd);
- waysx->fd=CloseFile(waysx->fd);
+ nodesx->fd=SlimUnmapFile(nodesx->fd);
+ segmentsx->fd=SlimUnmapFile(segmentsx->fd);
+ waysx->fd=SlimUnmapFile(waysx->fd);
 #endif
 
  /* Print the final message */
